@@ -22,6 +22,7 @@
 #include "gconf-sources.h"
 #include "gconf-locale.h"
 #include "gconfd.h"
+#include <string.h>
 #include <unistd.h>
 #include <time.h>
 #include <stdlib.h>
@@ -58,10 +59,12 @@ typedef struct _Listener Listener;
 
 struct _Listener {
   ConfigListener obj;
+  char *name;
 };
 
-static Listener* listener_new(ConfigListener obj);
-static void      listener_destroy(Listener* l);
+static Listener* listener_new (ConfigListener obj,
+                               const char    *name);
+static void      listener_destroy (Listener* l);
 
 /*
  * CORBA implementation of ConfigDatabase
@@ -78,7 +81,7 @@ impl_ConfigDatabase_add_listener(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return 0;
   
-  return gconf_database_add_listener (db, who, where);
+  return gconf_database_add_listener (db, who, NULL, where);
 }
 
 static void
@@ -258,10 +261,10 @@ impl_ConfigDatabase_set(PortableServer_Servant servant,
 }
 
 static void
-impl_ConfigDatabase_unset_with_locale(PortableServer_Servant servant,
-				      const CORBA_char * key,
-				      const CORBA_char * locale,
-				      CORBA_Environment * ev)
+impl_ConfigDatabase_unset_with_locale (PortableServer_Servant servant,
+                                       const CORBA_char * key,
+                                       const CORBA_char * locale,
+                                       CORBA_Environment * ev)
 {
   GConfDatabase *db = (GConfDatabase*) servant;
   GError* error = NULL;
@@ -473,10 +476,10 @@ impl_ConfigDatabase_all_dirs(PortableServer_Servant servant,
 }
 
 static void
-impl_ConfigDatabase_set_schema(PortableServer_Servant servant,
-			       const CORBA_char * key,
-			       const CORBA_char * schema_key,
-			       CORBA_Environment * ev)
+impl_ConfigDatabase_set_schema (PortableServer_Servant servant,
+                                const CORBA_char * key,
+                                const CORBA_char * schema_key,
+                                CORBA_Environment * ev)
 {
   GConfDatabase *db = (GConfDatabase*) servant;
   GError* error = NULL;
@@ -484,14 +487,17 @@ impl_ConfigDatabase_set_schema(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return;
   
-  gconf_database_set_schema(db, key, schema_key, &error);
-
-  gconf_set_exception(&error, ev);
+  gconf_database_set_schema (db, key,
+                             *schema_key != '\0' ?
+                             schema_key : NULL,
+                             &error);
+  
+  gconf_set_exception (&error, ev);
 }
 
 static void
-impl_ConfigDatabase_sync(PortableServer_Servant servant,
-			 CORBA_Environment * ev)
+impl_ConfigDatabase_sync (PortableServer_Servant servant,
+                          CORBA_Environment * ev)
 {
   GConfDatabase *db = (GConfDatabase*) servant;
   GError* error = NULL;
@@ -499,9 +505,9 @@ impl_ConfigDatabase_sync(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return;
   
-  gconf_database_sync(db, &error);
+  gconf_database_sync (db, &error);
 
-  gconf_set_exception(&error, ev);
+  gconf_set_exception (&error, ev);
 }
 
 static void
@@ -514,16 +520,16 @@ impl_ConfigDatabase_clear_cache(PortableServer_Servant servant,
   if (gconfd_check_in_shutdown (ev))
     return;
   
-  gconf_log(GCL_DEBUG, _("Received request to drop all cached data"));  
+  gconf_log (GCL_DEBUG, _("Received request to drop all cached data"));  
   
-  gconf_database_clear_cache(db, &error);
+  gconf_database_clear_cache (db, &error);
 
-  gconf_set_exception(&error, ev);
+  gconf_set_exception (&error, ev);
 }
 
 static void
-impl_ConfigDatabase_synchronous_sync(PortableServer_Servant servant,
-				     CORBA_Environment * ev)
+impl_ConfigDatabase_synchronous_sync (PortableServer_Servant servant,
+                                      CORBA_Environment * ev)
 {
   GConfDatabase *db = (GConfDatabase*) servant;
   GError* error = NULL;
@@ -534,9 +540,9 @@ impl_ConfigDatabase_synchronous_sync(PortableServer_Servant servant,
   gconf_log(GCL_DEBUG, _("Received request to sync synchronously"));
   
   
-  gconf_database_synchronous_sync(db, &error);
+  gconf_database_synchronous_sync (db, &error);
 
-  gconf_set_exception(&error, ev);
+  gconf_set_exception (&error, ev);
 }
 
 
@@ -701,6 +707,55 @@ impl_ConfigDatabase2_all_entries_with_schema_name(PortableServer_Servant servant
   g_slist_free(pairs);
 }
 
+static CORBA_unsigned_long
+impl_ConfigDatabase3_add_listener_with_properties (PortableServer_Servant servant,
+                                                   const CORBA_char *where,
+                                                   const ConfigListener who,
+                                                   const ConfigDatabase3_PropList *properties,
+                                                   CORBA_Environment *ev)
+{
+  GConfDatabase *db = (GConfDatabase*) servant;
+  const char *name = NULL;
+  int i;
+  
+  if (gconfd_check_in_shutdown (ev))
+    return 0;
+
+  i = 0;
+  while (i < (int) properties->_length)
+    {
+      if (strcmp (properties->_buffer[i].key, "name") == 0)
+        name = properties->_buffer[i].value;
+
+      ++i;
+    }
+  
+  return gconf_database_add_listener (db, who, name, where);
+}
+
+static void
+impl_ConfigDatabase3_recursive_unset (PortableServer_Servant servant,
+                                      const CORBA_char *key,
+                                      ConfigDatabase3_UnsetFlags flags,
+                                      CORBA_Environment *ev)
+{
+  GConfDatabase *db = (GConfDatabase*) servant;
+  GError *error;
+  GConfUnsetFlags gconf_flags;
+  
+  if (gconfd_check_in_shutdown (ev))
+    return;
+
+  gconf_flags = 0;
+  if (flags & ConfigDatabase3_UNSET_INCLUDING_SCHEMA_NAMES)
+    gconf_flags |= GCONF_UNSET_INCLUDING_SCHEMA_NAMES;
+  
+  error = NULL;
+  gconf_database_recursive_unset (db, key, NULL, gconf_flags, &error);
+
+  gconf_set_exception (&error, ev);
+}
+
 static PortableServer_ServantBase__epv base_epv = {
   NULL,
   NULL,
@@ -735,7 +790,13 @@ static POA_ConfigDatabase2__epv server2_epv = {
   impl_ConfigDatabase2_all_entries_with_schema_name
 };
 
-static POA_ConfigDatabase2__vepv poa_server_vepv = { &base_epv, &server_epv, &server2_epv };
+static POA_ConfigDatabase3__epv server3_epv = { 
+  NULL,
+  impl_ConfigDatabase3_add_listener_with_properties,
+  impl_ConfigDatabase3_recursive_unset
+};
+
+static POA_ConfigDatabase3__vepv poa_server_vepv = { &base_epv, &server_epv, &server2_epv, &server3_epv };
 
 static void gconf_database_really_sync(GConfDatabase* db);
 
@@ -744,7 +805,6 @@ gconf_database_new (GConfSources  *sources)
 {
   GConfDatabase* db;
   CORBA_Environment ev;
-  PortableServer_ObjectId* objid;
   
   db = g_new0 (GConfDatabase, 1);
 
@@ -753,16 +813,11 @@ gconf_database_new (GConfSources  *sources)
 
   CORBA_exception_init (&ev);
   
-  POA_ConfigDatabase2__init (&db->servant, &ev);
+  POA_ConfigDatabase3__init (&db->servant, &ev);
 
-  objid =
-    PortableServer_POA_activate_object(gconf_get_poa (),
-                                       &db->servant,
-                                       &ev);
-  
-  db->objref = PortableServer_POA_servant_to_reference(gconf_get_poa (),
-                                                       &db->servant,
-                                                       &ev);
+  db->objref = PortableServer_POA_servant_to_reference (gconf_get_poa (),
+                                                        &db->servant,
+                                                        &ev);
   if (CORBA_Object_is_nil(db->objref, &ev))
     {
       gconf_log(GCL_ERR,
@@ -771,8 +826,6 @@ gconf_database_new (GConfSources  *sources)
       exit (1);
     }
 
-  CORBA_free (objid);
-
   db->listeners = gconf_listeners_new();
 
   db->sources = sources;
@@ -807,7 +860,7 @@ gconf_database_free (GConfDatabase *db)
 
   CORBA_exception_free (&ev);
   
-  POA_ConfigDatabase2__fini (&db->servant, &ev);
+  POA_ConfigDatabase3__fini (&db->servant, &ev);
 
   CORBA_free (oid);
 
@@ -853,22 +906,26 @@ client_alive_predicate (const gchar* location,
 {
   Listener *l = listener_data;
   CORBA_Environment ev;
-
+  CORBA_boolean result;
+  
   CORBA_exception_init (&ev);
   
-  ConfigListener_ping (l->obj, &ev);
+  result = CORBA_Object_non_existent (l->obj, &ev);
 
   if (ev._major != CORBA_NO_EXCEPTION)
     {
-      gconf_log (GCL_DEBUG, "Removing stale listener %u, client not alive",
+      gconf_log (GCL_WARNING, "Exception from CORBA_Object_non_existent(), assuming stale listener %u",
                  cnxn_id);
       
       CORBA_exception_free (&ev);
 
-      return TRUE;
+      result = TRUE;
     }
-  else
-    return FALSE;
+
+  if (result)
+    gconf_log (GCL_DEBUG, "Dropping dead listener %s (%u), appears to be nonexistent", l->name, cnxn_id);
+  
+  return result;
 }
 
 void
@@ -888,7 +945,8 @@ gconf_database_sync_idle (GConfDatabase* db)
   db->sync_idle = 0;
 
   /* could have been added before reaching the
-     idle */
+   * idle
+   */
   if (db->sync_timeout != 0)
     {
       g_source_remove (db->sync_timeout);
@@ -970,23 +1028,27 @@ gconf_database_schedule_sync(GConfDatabase* db)
 CORBA_unsigned_long
 gconf_database_readd_listener   (GConfDatabase       *db,
                                  ConfigListener       who,
+                                 const char          *name,
                                  const gchar         *where)
 {
   Listener* l;
   guint cnxn;
-
+  
   gconfd_need_log_cleanup ();
   
   g_assert(db->listeners != NULL);
 
   db->last_access = time(NULL);
   
-  l = listener_new(who);
+  l = listener_new (who, name);
 
   cnxn = gconf_listeners_add (db->listeners, where, l,
                               (GFreeFunc)listener_destroy);
 
-  gconf_log (GCL_DEBUG, "Added listener %u", cnxn);
+  if (l->name == NULL)
+    l->name = g_strdup_printf ("%u", cnxn);
+  
+  gconf_log (GCL_DEBUG, "Added listener %s (%u)", l->name, cnxn);
   
   return cnxn;
 }
@@ -994,14 +1056,15 @@ gconf_database_readd_listener   (GConfDatabase       *db,
 CORBA_unsigned_long
 gconf_database_add_listener    (GConfDatabase       *db,
                                 ConfigListener       who,
+                                const char          *name,
                                 const gchar         *where)
 {
   GError *err;
-  CORBA_unsigned_long cnxn;
-
+  CORBA_unsigned_long cnxn;  
+  
   gconfd_need_log_cleanup ();
   
-  cnxn = gconf_database_readd_listener (db, who, where);
+  cnxn = gconf_database_readd_listener (db, who, name, where);
   
   err = NULL;
   if (!gconfd_logfile_change_listener (db, TRUE, cnxn,
@@ -1011,7 +1074,12 @@ gconf_database_add_listener    (GConfDatabase       *db,
        * Because it's likely the right thing for the client
        * app to simply continue.
        */
-      gconf_log (GCL_WARNING, _("Failed to log addition of listener (%s); will not be able to restore this listener on gconfd restart, resulting in unreliable notification of configuration changes."), err->message);
+      gconf_log (GCL_WARNING,
+		 _("Failed to log addition of listener %s (%s);"
+		   "will not be able to restore this listener on "
+		   "gconfd restart, resulting in unreliable "
+		   "notification of configuration changes."),
+		 name, err->message);
       g_error_free (err);
     }
   
@@ -1042,6 +1110,11 @@ gconf_database_remove_listener (GConfDatabase       *db,
                  (gulong) cnxn);
       return;
     }
+  else
+    {
+      gconf_log (GCL_DEBUG, "Name of listener %u is %s",
+                 (guint) cnxn, l->name);
+    }
   
   err = NULL;
   if (!gconfd_logfile_change_listener (db, FALSE, cnxn,
@@ -1053,7 +1126,7 @@ gconf_database_remove_listener (GConfDatabase       *db,
     }
   
   /* calls destroy notify */
-  gconf_listeners_remove(db->listeners, cnxn);
+  gconf_listeners_remove (db->listeners, cnxn);
 }
 
 typedef struct _ListenerNotifyClosure ListenerNotifyClosure;
@@ -1088,8 +1161,8 @@ notify_listeners_cb(GConfListeners* listeners,
   
   if(closure->ev._major != CORBA_NO_EXCEPTION) 
     {
-      gconf_log (GCL_DEBUG, "Failed to notify listener %u, removing: %s", 
-                 cnxn_id, CORBA_exception_id (&closure->ev));
+      gconf_log (GCL_DEBUG, "Failed to notify listener %s (%u), removing: %s", 
+                 l->name, cnxn_id, CORBA_exception_id (&closure->ev));
       CORBA_exception_free (&closure->ev);
       
       /* Dead listeners need to be forgotten */
@@ -1097,8 +1170,8 @@ notify_listeners_cb(GConfListeners* listeners,
     }
   else
     {
-      gconf_log (GCL_DEBUG, "Notified listener %u of change to key `%s'",
-                 cnxn_id, all_above_key);
+      gconf_log (GCL_DEBUG, "Notified listener %s (%u) of change to key `%s'",
+                 l->name, cnxn_id, all_above_key);
     }
 }
 
@@ -1270,7 +1343,12 @@ gconf_database_unset (GConfDatabase      *db,
       GConfValue* def_value;
       const gchar* locale_list[] = { NULL, NULL };
       gboolean is_writable = TRUE;
-      
+
+      /* This is a somewhat dubious optimization
+       * that assumes that if the unset was successful
+       * the default value is the new value. Which is
+       * safe for now, I _think_
+       */
       locale_list[0] = locale;
       def_value = gconf_database_query_default_value(db,
                                                      key,
@@ -1300,6 +1378,97 @@ gconf_database_unset (GConfDatabase      *db,
     }
 }
 
+void
+gconf_database_recursive_unset (GConfDatabase      *db,
+                                const gchar        *key,
+                                const gchar        *locale,
+                                GConfUnsetFlags     flags,
+                                GError            **err)
+{
+  ConfigValue* val;
+  GError* error = NULL;
+  GSList *notifies;
+  GSList *tmp;
+  
+  g_return_if_fail (err == NULL || *err == NULL);
+  
+  g_assert (db->listeners != NULL);
+  
+  db->last_access = time (NULL);
+  
+  gconf_log (GCL_DEBUG, "Received request to recursively unset key \"%s\"", key);
+
+  notifies = NULL;
+  gconf_sources_recursive_unset (db->sources, key, locale,
+                                 flags, &notifies, &error);
+
+  /* We return the error but go ahead and finish the unset.
+   * We're just returning the first error seen during the
+   * unset process.
+   */
+  if (error != NULL)
+    {
+      gconf_log (GCL_ERR, _("Error unsetting \"%s\": %s"),
+                 key, error->message);
+
+      if (err)
+        *err = error;
+      else
+        g_error_free (error);
+
+      error = NULL;
+    }
+  
+  tmp = notifies;
+  while (tmp != NULL)
+    {
+      GConfValue* new_value;
+      const gchar* locale_list[] = { NULL, NULL };
+      gboolean is_writable = TRUE;
+      gboolean is_default = TRUE;
+      char *notify_key = tmp->data;
+      
+
+      locale_list[0] = locale;
+      new_value = gconf_database_query_value (db,
+                                              notify_key,
+                                              locale_list,
+                                              TRUE,
+                                              NULL,
+                                              &is_default,
+                                              &is_writable,
+                                              &error);
+
+      if (error)
+        gconf_log (GCL_ERR, _("Error getting new value for \"%s\": %s"),
+                   notify_key, error->message);
+      g_propagate_error (err, error);
+      error = NULL;
+      
+      if (new_value != NULL)
+        {
+          val = corba_value_from_gconf_value (new_value);
+          gconf_value_free (new_value);
+        }
+      else
+        {
+          val = invalid_corba_value();
+        }
+          
+      gconf_database_schedule_sync (db);
+
+      gconf_database_notify_listeners (db, notify_key, val,
+                                       is_default, is_writable);
+      
+      CORBA_free (val);
+      g_free (notify_key);
+      
+      tmp = tmp->next;
+    }
+
+  g_slist_free (notifies);
+}
+
 gboolean
 gconf_database_dir_exists  (GConfDatabase  *db,
                             const gchar    *dir,
@@ -1406,23 +1575,23 @@ void
 gconf_database_set_schema (GConfDatabase  *db,
                            const gchar    *key,
                            const gchar    *schema_key,
-                           GError    **err)
+                           GError        **err)
 {
-  g_return_if_fail(err == NULL || *err == NULL);
-  g_assert(db->listeners != NULL);
+  g_return_if_fail (err == NULL || *err == NULL);
+  g_assert (db->listeners != NULL);
   
-  db->last_access = time(NULL);
+  db->last_access = time (NULL);
   
-  gconf_sources_set_schema(db->sources, key, schema_key, err);
+  gconf_sources_set_schema (db->sources, key, schema_key, err);
 
   if (err && *err != NULL)
     {
-      gconf_log(GCL_ERR, _("Error setting schema for `%s': %s"),
+      gconf_log (GCL_ERR, _("Error setting schema for `%s': %s"),
                 key, (*err)->message);
     }
   else
     {
-      gconf_database_schedule_sync(db);
+      gconf_database_schedule_sync (db);
     }
 }
 
@@ -1505,6 +1674,9 @@ listener_save_foreach (const gchar* location,
   CORBA_Environment ev;
   gchar *ior;
   gchar *s;
+
+  gconf_log (GCL_DEBUG, "Saving listener %s (%u) to log file", l->name,
+             (guint) cnxn_id);
   
   s = g_strdup_printf ("ADD %u %s ", cnxn_id, fd->db_name);
 
@@ -1602,27 +1774,32 @@ gconfd_locale_cache_drop(void)
  */
 
 static Listener* 
-listener_new(ConfigListener obj)
+listener_new (ConfigListener obj,
+              const char    *name)
 {
   Listener* l;
   CORBA_Environment ev;
 
-  CORBA_exception_init(&ev);
+  CORBA_exception_init (&ev);
 
-  l = g_new0(Listener, 1);
-
-  l->obj = CORBA_Object_duplicate(obj, &ev);
+  l = g_new0 (Listener, 1);
 
+  l->obj = CORBA_Object_duplicate (obj, &ev);
+  l->name = g_strdup (name);
+  
   return l;
 }
 
 static void      
-listener_destroy(Listener* l)
-
+listener_destroy (Listener* l)
 {  
   CORBA_Environment ev;
 
-  CORBA_exception_init(&ev);
-  CORBA_Object_release(l->obj, &ev);
-  g_free(l);
+  CORBA_exception_init (&ev);
+  CORBA_Object_release (l->obj, &ev);
+  g_free (l->name);
+  g_free (l);
 }
+
+
+
