@@ -977,46 +977,30 @@ gconf_string_to_gulong(const gchar* str)
 }
 
 gboolean
-gconf_string_to_double(const gchar* str, gdouble* retloc)
+gconf_string_to_double(const gchar* str,
+                       gdouble*     retloc)
 {
-  int res;
-  char *old_locale;
+  char *end;
 
-  /* make sure we write values to files in a consistent manner */
-  old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-  setlocale (LC_NUMERIC, "C");
-
-  *retloc = 0.0;
-  res = sscanf (str, "%lf", retloc);
-
-  setlocale (LC_NUMERIC, old_locale);
-  g_free (old_locale);
-
-  if (res == 1)
-    return TRUE;
+  errno = 0;
+  *retloc = g_ascii_strtod (str, &end);
+  if (end == str || errno != 0)
+    {
+      *retloc = 0.0;
+      return FALSE;
+    }
   else
-    return FALSE;
+    return TRUE;
 }
 
 gchar*
-gconf_double_to_string(gdouble val)
+gconf_double_to_string (gdouble val)
 {
-  char str[101 + DBL_DIG];
-  char *old_locale;
+  char str[G_ASCII_DTOSTR_BUF_SIZE];
 
-  /* make sure we write values to files in a consistent manner */
-  old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-  setlocale (LC_NUMERIC, "C");
+  g_ascii_dtostr (str, G_ASCII_DTOSTR_BUF_SIZE, val);
   
-  if (fabs (val) < 1e9 && fabs (val) > 1e-5)
-    g_snprintf (str, 100 + DBL_DIG, "%.*g", DBL_DIG, val);
-  else
-    g_snprintf (str, 100 + DBL_DIG, "%f", val);
-
-  setlocale (LC_NUMERIC, old_locale);
-  g_free (old_locale);
-  
-  return g_strdup(str);
+  return g_strdup (str);
 }
 
 const gchar*
@@ -1378,13 +1362,9 @@ gconf_value_list_to_primitive_list_destructive(GConfValue* val,
 
   g_assert(gconf_value_get_list_type(val) == list_type);
       
-  retval = gconf_value_get_list(val);
-
-  /* Cheating the API to avoid a list copy; set this to NULL to
-         avoid destroying the list */
-  val->d.list_data.list = NULL;
+  retval = gconf_value_steal_list (val);
       
-  gconf_value_free(val);
+  gconf_value_free (val);
   val = NULL;
       
   {
@@ -1403,9 +1383,12 @@ gconf_value_list_to_primitive_list_destructive(GConfValue* val,
         switch (list_type)
           {
           case GCONF_VALUE_INT:
-          case GCONF_VALUE_BOOL:
             tmp->data = GINT_TO_POINTER(gconf_value_get_int(elem));
             break;
+
+          case GCONF_VALUE_BOOL:
+            tmp->data = GINT_TO_POINTER(gconf_value_get_bool(elem));
+            break;
                 
           case GCONF_VALUE_FLOAT:
             {
@@ -1416,19 +1399,11 @@ gconf_value_list_to_primitive_list_destructive(GConfValue* val,
             break;
 
           case GCONF_VALUE_STRING:
-            {
-              /* Cheat again, and steal the string from the value */
-              tmp->data = elem->d.string_data;
-              elem->d.string_data = NULL;
-            }
+            tmp->data = gconf_value_steal_string (elem);
             break;
 
           case GCONF_VALUE_SCHEMA:
-            {
-              /* and also steal the schema... */
-              tmp->data = elem->d.schema_data;
-              elem->d.schema_data = NULL;
-            }
+            tmp->data = gconf_value_steal_schema (elem);
             break;
                 
           default:
@@ -1462,9 +1437,7 @@ primitive_value(gpointer retloc, GConfValue* val)
 
     case GCONF_VALUE_STRING:
       {
-        *((gchar**)retloc) = val->d.string_data;
-        /* cheat and steal the string to avoid a copy */
-        val->d.string_data = NULL;
+        *((gchar**)retloc) = gconf_value_steal_string (val);
       }
       break;
 
@@ -1473,7 +1446,7 @@ primitive_value(gpointer retloc, GConfValue* val)
       break;
 
     case GCONF_VALUE_SCHEMA:
-      *((GConfSchema**)retloc) = gconf_value_get_schema(val);
+      *((GConfSchema**)retloc) = gconf_value_steal_schema(val);
       break;
       
     default:
@@ -2412,8 +2385,8 @@ open_empty_locked_file (const gchar *directory,
 }
 
 static ConfigServer
-read_current_server (const gchar *iorfile,
-                     gboolean     warn_if_fail)
+read_current_server_and_set_warning (const gchar *iorfile,
+                                     GString     *warning)
 {
   FILE *fp;
   
@@ -2421,9 +2394,10 @@ read_current_server (const gchar *iorfile,
           
   if (fp == NULL)
     {
-      if (warn_if_fail)
-        gconf_log (GCL_WARNING, _("IOR file '%s' not opened successfully, no gconfd located: %s"),
-                   iorfile, g_strerror (errno));
+      if (warning)
+        g_string_append_printf (warning,
+                                _("IOR file '%s' not opened successfully, no gconfd located: %s"),
+                                iorfile, g_strerror (errno));
 
       return CORBA_OBJECT_NIL;
     }
@@ -2450,10 +2424,10 @@ read_current_server (const gchar *iorfile,
           str[2] == 'n' &&
           str[3] == 'e')
         {
-          if (warn_if_fail)
-            gconf_log (GCL_WARNING,
-                       _("gconftool or other non-gconfd process has the lock file '%s'"),
-                       iorfile);          
+          if (warning)
+            g_string_append_printf (warning,
+                                    _("gconftool or other non-gconfd process has the lock file '%s'"),
+                                    iorfile); 
         }
       else /* file contains daemon IOR */
         {
@@ -2467,9 +2441,9 @@ read_current_server (const gchar *iorfile,
 
           if (orb == NULL)
             {
-              if (warn_if_fail)
-                gconf_log (GCL_WARNING,
-                           _("couldn't contact ORB to resolve existing gconfd object reference"));
+              if (warning)
+                g_string_append_printf (warning,
+                                        _("couldn't contact ORB to resolve existing gconfd object reference"));
               return CORBA_OBJECT_NIL;
             }
                   
@@ -2477,10 +2451,10 @@ read_current_server (const gchar *iorfile,
           CORBA_exception_free (&ev);
 
           if (server == CORBA_OBJECT_NIL &&
-              warn_if_fail)
-            gconf_log (GCL_WARNING,
-                       _("Failed to convert IOR '%s' to an object reference"),
-                       str);
+              warning)
+            g_string_append_printf (warning,
+                                    _("Failed to convert IOR '%s' to an object reference"),
+                                    str);
           
           return server;
         }
@@ -2489,6 +2463,28 @@ read_current_server (const gchar *iorfile,
     }
 }
 
+static ConfigServer
+read_current_server (const gchar *iorfile,
+                     gboolean     warn_if_fail)
+{
+  GString *warning;
+  ConfigServer server;
+  
+  if (warn_if_fail)
+    warning = g_string_new (NULL);
+  else
+    warning = NULL;
+
+  server = read_current_server_and_set_warning (iorfile, warning);
+
+  if (warning->len > 0)
+    gconf_log (GCL_WARNING, "%s", warning->str);
+
+  g_string_free (warning, TRUE);
+
+  return server;
+}
+
 GConfLock*
 gconf_get_lock_or_current_holder (const gchar  *lock_directory,
                                   ConfigServer *current_server,
@@ -2692,13 +2688,14 @@ gconf_release_lock (GConfLock *lock,
  * of the sort; it just reads it. It does do the object_to_string
  */
 ConfigServer
-gconf_get_current_lock_holder  (const gchar *lock_directory)
+gconf_get_current_lock_holder  (const gchar *lock_directory,
+                                GString     *failure_log)
 {
   char *iorfile;
   ConfigServer server;
 
   iorfile = g_strconcat (lock_directory, "/ior", NULL);
-  server = read_current_server (iorfile, FALSE);
+  server = read_current_server_and_set_warning (iorfile, failure_log);
   g_free (iorfile);
   return server;
 }
@@ -2838,7 +2835,10 @@ gconf_activate_server (gboolean  start_if_not_found,
   char *argv[3];
   char *gconfd_dir;
   char *lock_dir;
+  GString *failure_log;
   CORBA_Environment ev;
+
+  failure_log = g_string_new (NULL);
   
   gconfd_dir = gconf_get_daemon_dir ();
   
@@ -2847,9 +2847,10 @@ gconf_activate_server (gboolean  start_if_not_found,
                gconfd_dir, g_strerror (errno));
 
   g_free (gconfd_dir);
-  
+
+  g_string_append (failure_log, " 1: ");
   lock_dir = gconf_get_lock_dir ();
-  server = gconf_get_current_lock_holder (lock_dir);
+  server = gconf_get_current_lock_holder (lock_dir, failure_log);
   g_free (lock_dir);
 
   /* Confirm server exists */
@@ -2860,13 +2861,22 @@ gconf_activate_server (gboolean  start_if_not_found,
       ConfigServer_ping (server, &ev);
       
       if (ev._major != CORBA_NO_EXCEPTION)
-        server = CORBA_OBJECT_NIL;
+        {
+          server = CORBA_OBJECT_NIL;
+
+          g_string_append_printf (failure_log,
+                                  _("Server ping error: %s"),
+                                  CORBA_exception_id (&ev));
+        }
     }
 
   CORBA_exception_free (&ev);
   
   if (server != CORBA_OBJECT_NIL)
-    return server;
+    {
+      g_string_free (failure_log, TRUE);
+      return server;
+    }
   
   if (start_if_not_found)
     {
@@ -2912,8 +2922,9 @@ gconf_activate_server (gboolean  start_if_not_found,
       /* Block until server starts up */
       read (p[0], buf, 1);
 
+      g_string_append (failure_log, " 2: ");
       lock_dir = gconf_get_lock_dir ();
-      server = gconf_get_current_lock_holder (lock_dir);
+      server = gconf_get_current_lock_holder (lock_dir, failure_log);
       g_free (lock_dir);
     }
   
@@ -2924,10 +2935,47 @@ gconf_activate_server (gboolean  start_if_not_found,
     g_set_error (error,
                  GCONF_ERROR,
                  GCONF_ERROR_NO_SERVER,
-                 _("Failed to contact configuration server; some possible causes are:\na) you have an existing configuration server (gconfd) running, but it isn't reachable from here - if you're logged in from two machines at once, you may need to enable TCP networking for ORBit by putting the line \"ORBIIOPIPv4=1\" in /etc/orbitrc\nb) you have stale locks in your NFS-mounted home directory due to a system crash, try removing ~/.gconf/*.lock and ~/.gconfd/lock if you are sure no gconfd processes are running on any machine using your home dir"));
+                 _("Failed to contact configuration server; some possible causes are that you need to enable TCP/IP networking for ORBit, or you have stale NFS locks due to a system crash. See http://www.gnome.org/projects/gconf/ for information. (Details - %s)"),
+                 failure_log->len > 0 ? failure_log->str : _("none"));
 
+  g_string_free (failure_log, TRUE);
+  
   close (p[0]);
   close (p[1]);
   
   return server;
 }
+
+gboolean
+gconf_CORBA_Object_equal (gconstpointer a, gconstpointer b)
+{
+  CORBA_Environment ev;
+  CORBA_Object _obj_a = (gpointer)a;
+  CORBA_Object _obj_b = (gpointer)b;
+  gboolean retval;
+
+  CORBA_exception_init (&ev);
+  retval = CORBA_Object_is_equivalent(_obj_a, _obj_b, &ev);
+  CORBA_exception_free (&ev);
+
+  return retval;
+}
+
+guint
+gconf_CORBA_Object_hash (gconstpointer key)
+{
+  CORBA_Environment ev;
+  CORBA_Object _obj = (gpointer)key;
+  CORBA_unsigned_long retval;
+
+  CORBA_exception_init (&ev);
+  retval = CORBA_Object_hash(_obj, G_MAXUINT, &ev);
+  CORBA_exception_free (&ev);
+
+  return retval;
+}
+
+
+
+
+
