@@ -121,6 +121,10 @@ GConfValue* gconf_value_new_pair_from_string (GConfValueType car_type,
                                               const gchar* str,
 					      GError** err);
 
+GSList*      gconf_value_steal_list   (GConfValue *value);
+GConfSchema* gconf_value_steal_schema (GConfValue *value);
+char*        gconf_value_steal_string (GConfValue *value);
+
 /* These are a hack to encode values into strings and ship them over CORBA,
  * necessary for obscure reasons (ORBit doesn't like recursive datatypes yet)
  */
@@ -184,7 +188,8 @@ gboolean   gconf_release_lock (GConfLock    *lock,
 GConfLock* gconf_get_lock_or_current_holder (const gchar  *lock_directory,
                                              ConfigServer *current_server,
                                              GError      **err);
-ConfigServer gconf_get_current_lock_holder  (const gchar *lock_directory);
+ConfigServer gconf_get_current_lock_holder  (const gchar *lock_directory,
+                                             GString     *failure_log);
 
 GError*  gconf_error_new  (GConfError en,
                            const gchar* format, ...) G_GNUC_PRINTF (2, 3);
@@ -222,42 +227,19 @@ gboolean gconf_engine_recursive_unset (GConfEngine      *engine,
                                        GConfUnsetFlags   flags,
                                        GError          **err);
 
-static inline gboolean
-gconf_CORBA_Object_equal (gconstpointer a, gconstpointer b)
-{
-  CORBA_Environment ev;
-  CORBA_Object _obj_a = (gpointer)a;
-  CORBA_Object _obj_b = (gpointer)b;
-  gboolean retval;
+gboolean gconf_CORBA_Object_equal (gconstpointer a,
+                                   gconstpointer b);
+guint    gconf_CORBA_Object_hash  (gconstpointer key);
 
-  CORBA_exception_init (&ev);
-  retval = CORBA_Object_is_equivalent(_obj_a, _obj_b, &ev);
-  CORBA_exception_free (&ev);
-
-  return retval;
-}
-
-static inline guint
-gconf_CORBA_Object_hash(gconstpointer key)
-{
-  CORBA_Environment ev;
-  CORBA_Object _obj = (gpointer)key;
-  CORBA_unsigned_long retval;
-
-  CORBA_exception_init (&ev);
-  retval = CORBA_Object_hash(_obj, G_MAXUINT, &ev);
-  CORBA_exception_free (&ev);
-
-  return retval;
-}
 
 /* FIXME move to public eventually */
 GConfEntry* gconf_entry_copy (const GConfEntry *src);
 gboolean    gconf_entry_equal (const GConfEntry *a,
                                const GConfEntry *b);
+void        gconf_entry_ref   (GConfEntry *entry);
+void        gconf_entry_unref (GConfEntry *entry);
 int         gconf_value_compare (const GConfValue *value_a,
                                  const GConfValue *value_b);
-
 #endif /* GCONF_ENABLE_INTERNALS */
 
 #endif /* GCONF_GCONF_INTERNALS_H */
