@@ -25,6 +25,27 @@
 #include <string.h>
 #include <stdlib.h>
 
+typedef struct {
+  GConfValueType type;
+  union {
+    gchar* string_data;
+    gint int_data;
+    gboolean bool_data;
+    gdouble float_data;
+    GConfSchema* schema_data;
+    struct {
+      GConfValueType type;
+      GSList* list;
+    } list_data;
+    struct {
+      GConfValue* car;
+      GConfValue* cdr;
+    } pair_data;
+  } d;
+} GConfRealValue;
+
+#define REAL_VALUE(x) ((GConfRealValue*)(x))
+
 static void
 set_string(gchar** dest, const gchar* src)
 {
@@ -46,12 +67,13 @@ gconf_value_new(GConfValueType type)
   g_return_val_if_fail(GCONF_VALUE_TYPE_VALID(type), NULL);
   
   /* Probably want to use mem chunks here eventually. */
-  value = g_new0(GConfValue, 1);
+  value = (GConfValue*) g_new0 (GConfRealValue, 1);
 
   value->type = type;
 
   /* the g_new0() is important: sets list type to invalid, NULLs all
-     pointers */
+   * pointers
+   */
   
   return value;
 }
@@ -681,28 +703,30 @@ copy_value_list(GSList* list)
 }
 
 GConfValue* 
-gconf_value_copy(const GConfValue* src)
+gconf_value_copy (const GConfValue* src)
 {
-  GConfValue* dest;
-
+  GConfRealValue *dest;
+  GConfRealValue *real;
+  
   g_return_val_if_fail(src != NULL, NULL);
 
-  dest = gconf_value_new(src->type);
+  real = REAL_VALUE (src);
+  dest = REAL_VALUE (gconf_value_new (src->type));
 
-  switch (src->type)
+  switch (real->type)
     {
     case GCONF_VALUE_INT:
     case GCONF_VALUE_FLOAT:
     case GCONF_VALUE_BOOL:
     case GCONF_VALUE_INVALID:
-      dest->d = src->d;
+      dest->d = real->d;
       break;
     case GCONF_VALUE_STRING:
-      set_string(&dest->d.string_data, src->d.string_data);
+      set_string(&dest->d.string_data, real->d.string_data);
       break;
     case GCONF_VALUE_SCHEMA:
-      if (src->d.schema_data)
-        dest->d.schema_data = gconf_schema_copy(src->d.schema_data);
+      if (real->d.schema_data)
+        dest->d.schema_data = gconf_schema_copy(real->d.schema_data);
       else
         dest->d.schema_data = NULL;
       break;
@@ -711,22 +735,22 @@ gconf_value_copy(const GConfValue* src)
       {
         GSList* copy;
 
-        copy = copy_value_list(src->d.list_data.list);
+        copy = copy_value_list(real->d.list_data.list);
 
         dest->d.list_data.list = copy;
-        dest->d.list_data.type = src->d.list_data.type;
+        dest->d.list_data.type = real->d.list_data.type;
       }
       break;
       
     case GCONF_VALUE_PAIR:
 
-      if (src->d.pair_data.car)
-        dest->d.pair_data.car = gconf_value_copy(src->d.pair_data.car);
+      if (real->d.pair_data.car)
+        dest->d.pair_data.car = gconf_value_copy(real->d.pair_data.car);
       else
         dest->d.pair_data.car = NULL;
 
-      if (src->d.pair_data.cdr)
-        dest->d.pair_data.cdr = gconf_value_copy(src->d.pair_data.cdr);
+      if (real->d.pair_data.cdr)
+        dest->d.pair_data.cdr = gconf_value_copy(real->d.pair_data.cdr);
       else
         dest->d.pair_data.cdr = NULL;
 
@@ -736,17 +760,21 @@ gconf_value_copy(const GConfValue* src)
       g_assert_not_reached();
     }
   
-  return dest;
+  return (GConfValue*) dest;
 }
 
 static void
 gconf_value_free_list(GConfValue* value)
 {
   GSList* tmp;
+  GConfRealValue *real;
+  
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_LIST);
+
+  real = REAL_VALUE (value);
   
-  tmp = value->d.list_data.list;
+  tmp = real->d.list_data.list;
 
   while (tmp != NULL)
     {
@@ -754,35 +782,39 @@ gconf_value_free_list(GConfValue* value)
       
       tmp = g_slist_next(tmp);
     }
-  g_slist_free(value->d.list_data.list);
+  g_slist_free(real->d.list_data.list);
 
-  value->d.list_data.list = NULL;
+  real->d.list_data.list = NULL;
 }
 
 void 
 gconf_value_free(GConfValue* value)
 {
+  GConfRealValue *real;
+  
   g_return_if_fail(value != NULL);
 
-  switch (value->type)
+  real = REAL_VALUE (value);
+  
+  switch (real->type)
     {
     case GCONF_VALUE_STRING:
-      if (value->d.string_data != NULL)
-        g_free(value->d.string_data);
+      if (real->d.string_data != NULL)
+        g_free(real->d.string_data);
       break;
     case GCONF_VALUE_SCHEMA:
-      if (value->d.schema_data != NULL)
-        gconf_schema_free(value->d.schema_data);
+      if (real->d.schema_data != NULL)
+        gconf_schema_free(real->d.schema_data);
       break;
     case GCONF_VALUE_LIST:
       gconf_value_free_list(value);
       break;
     case GCONF_VALUE_PAIR:
-      if (value->d.pair_data.car != NULL)
-        gconf_value_free(value->d.pair_data.car);
+      if (real->d.pair_data.car != NULL)
+        gconf_value_free(real->d.pair_data.car);
 
-      if (value->d.pair_data.cdr != NULL)
-        gconf_value_free(value->d.pair_data.cdr);
+      if (real->d.pair_data.cdr != NULL)
+        gconf_value_free(real->d.pair_data.cdr);
       break;
     default:
       break;
@@ -791,22 +823,158 @@ gconf_value_free(GConfValue* value)
   g_free(value);
 }
 
+const char*
+gconf_value_get_string (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_STRING, NULL);
+  
+  return REAL_VALUE (value)->d.string_data;
+}
+
+char*
+gconf_value_steal_string (GConfValue *value)
+{
+  char *string;
+  GConfRealValue *real;
+  
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_STRING, NULL);
+
+  real = REAL_VALUE (value);
+
+  string = real->d.string_data;
+  real->d.string_data = NULL;
+
+  return string;
+}
+
+int
+gconf_value_get_int (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, 0);
+  g_return_val_if_fail (value->type == GCONF_VALUE_INT, 0);
+  
+  return REAL_VALUE (value)->d.int_data;
+}
+
+double
+gconf_value_get_float (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, 0.0);
+  g_return_val_if_fail (value->type == GCONF_VALUE_FLOAT, 0.0);
+  
+  return REAL_VALUE (value)->d.float_data;
+}
+
+GConfValueType
+gconf_value_get_list_type (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, GCONF_VALUE_INVALID);
+  g_return_val_if_fail (value->type == GCONF_VALUE_LIST, GCONF_VALUE_INVALID);
+  
+  return REAL_VALUE (value)->d.list_data.type;
+}
+
+GSList*
+gconf_value_get_list (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_LIST, NULL);
+
+  return REAL_VALUE (value)->d.list_data.list;
+}
+
+GSList*
+gconf_value_steal_list (GConfValue *value)
+{
+  GSList *list;
+  GConfRealValue *real;
+  
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_LIST, NULL);
+
+  real = REAL_VALUE (value);
+
+  list = real->d.list_data.list;
+  real->d.list_data.list = NULL;
+  return list;
+}
+
+GConfValue*
+gconf_value_get_car (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_PAIR, NULL);
+  
+  return REAL_VALUE (value)->d.pair_data.car;
+}
+
+GConfValue*
+gconf_value_get_cdr (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_PAIR, NULL);
+  
+  return REAL_VALUE (value)->d.pair_data.cdr;
+}
+
+
+gboolean
+gconf_value_get_bool (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, FALSE);
+  g_return_val_if_fail (value->type == GCONF_VALUE_BOOL, FALSE);
+  
+  return REAL_VALUE (value)->d.bool_data;
+}
+
+GConfSchema*
+gconf_value_get_schema (const GConfValue *value)
+{
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_SCHEMA, NULL);
+  
+  return REAL_VALUE (value)->d.schema_data;
+}
+
+GConfSchema*
+gconf_value_steal_schema (GConfValue *value)
+{
+  GConfSchema *schema;
+  GConfRealValue *real;
+  
+  g_return_val_if_fail (value != NULL, NULL);
+  g_return_val_if_fail (value->type == GCONF_VALUE_SCHEMA, NULL);
+
+  real = REAL_VALUE (value);
+
+  schema = real->d.schema_data;
+  real->d.schema_data = NULL;
+
+  return schema;
+}
+
 void        
 gconf_value_set_int(GConfValue* value, gint the_int)
 {
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_INT);
 
-  value->d.int_data = the_int;
+  REAL_VALUE (value)->d.int_data = the_int;
 }
 
 void        
 gconf_value_set_string(GConfValue* value, const gchar* the_str)
 {
+  GConfRealValue *real;
+
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_STRING);
 
-  set_string(&value->d.string_data, the_str);
+  real = REAL_VALUE (value);
+  
+  set_string(&real->d.string_data, the_str);
 }
 
 void        
@@ -815,7 +983,7 @@ gconf_value_set_float(GConfValue* value, gdouble the_float)
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_FLOAT);
 
-  value->d.float_data = the_float;
+  REAL_VALUE (value)->d.float_data = the_float;
 }
 
 void        
@@ -824,32 +992,40 @@ gconf_value_set_bool(GConfValue* value, gboolean the_bool)
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_BOOL);
 
-  value->d.bool_data = the_bool;
+  REAL_VALUE (value)->d.bool_data = the_bool;
 }
 
 void       
 gconf_value_set_schema(GConfValue* value, const GConfSchema* sc)
 {
+  GConfRealValue *real;
+  
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_SCHEMA);
-  
-  if (value->d.schema_data != NULL)
-    gconf_schema_free(value->d.schema_data);
 
-  value->d.schema_data = gconf_schema_copy(sc);
+  real = REAL_VALUE (value);
+  
+  if (real->d.schema_data != NULL)
+    gconf_schema_free (real->d.schema_data);
+
+  real->d.schema_data = gconf_schema_copy (sc);
 }
 
 void        
 gconf_value_set_schema_nocopy(GConfValue* value, GConfSchema* sc)
 {
+  GConfRealValue *real;
+  
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_SCHEMA);
   g_return_if_fail(sc != NULL);
-  
-  if (value->d.schema_data != NULL)
-    gconf_schema_free(value->d.schema_data);
 
-  value->d.schema_data = sc;
+  real = REAL_VALUE (value);
+  
+  if (real->d.schema_data != NULL)
+    gconf_schema_free (real->d.schema_data);
+
+  real->d.schema_data = sc;
 }
 
 void
@@ -861,13 +1037,17 @@ gconf_value_set_car(GConfValue* value, const GConfValue* car)
 void
 gconf_value_set_car_nocopy(GConfValue* value, GConfValue* car)
 {
+  GConfRealValue *real;
+  
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_PAIR);
-  
-  if (value->d.pair_data.car != NULL)
-    gconf_value_free(value->d.pair_data.car);
 
-  value->d.pair_data.car = car;
+  real = REAL_VALUE (value);
+  
+  if (real->d.pair_data.car != NULL)
+    gconf_value_free (real->d.pair_data.car);
+
+  real->d.pair_data.car = car;
 }
 
 void
@@ -879,65 +1059,86 @@ gconf_value_set_cdr(GConfValue* value, const GConfValue* cdr)
 void
 gconf_value_set_cdr_nocopy(GConfValue* value, GConfValue* cdr)
 {
+  GConfRealValue *real;
+  
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_PAIR);
-  
-  if (value->d.pair_data.cdr != NULL)
-    gconf_value_free(value->d.pair_data.cdr);
 
-  value->d.pair_data.cdr = cdr;
+  real = REAL_VALUE (value);
+  
+  if (real->d.pair_data.cdr != NULL)
+    gconf_value_free (real->d.pair_data.cdr);
+
+  real->d.pair_data.cdr = cdr;
 }
 
 void
-gconf_value_set_list_type(GConfValue* value,
+gconf_value_set_list_type (GConfValue    *value,
                            GConfValueType type)
 {
+  GConfRealValue *real;
+  
   g_return_if_fail(value != NULL);
   g_return_if_fail(value->type == GCONF_VALUE_LIST);
   g_return_if_fail(type != GCONF_VALUE_LIST);
   g_return_if_fail(type != GCONF_VALUE_PAIR);
-  /* If the list is non-NULL either we already have the right
-     type, or we shouldn't be changing it without deleting
-     the list first. */
-  g_return_if_fail(value->d.list_data.list == NULL);
 
-  value->d.list_data.type = type;
+  real = REAL_VALUE (value);
+  
+  /* If the list is non-NULL either we already have the right
+   * type, or we shouldn't be changing it without deleting
+   * the list first.
+   */
+  g_return_if_fail (real->d.list_data.list == NULL);
+
+  real->d.list_data.type = type;
 }
 
 void
-gconf_value_set_list_nocopy(GConfValue* value,
+gconf_value_set_list_nocopy (GConfValue* value,
                              GSList* list)
 {
-  g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == GCONF_VALUE_LIST);
-  g_return_if_fail(value->d.list_data.type != GCONF_VALUE_INVALID);
+  GConfRealValue *real;
   
-  if (value->d.list_data.list)
-    gconf_value_free_list(value);
+  g_return_if_fail (value != NULL);
+  g_return_if_fail (value->type == GCONF_VALUE_LIST);
 
-  value->d.list_data.list = list;
+  real = REAL_VALUE (value);
+  
+  g_return_if_fail (real->d.list_data.type != GCONF_VALUE_INVALID);
+  
+  if (real->d.list_data.list)
+    gconf_value_free_list (value);
+
+  real->d.list_data.list = list;
 }
 
 void
 gconf_value_set_list       (GConfValue* value,
                             GSList* list)
 {
-  g_return_if_fail(value != NULL);
-  g_return_if_fail(value->type == GCONF_VALUE_LIST);
-  g_return_if_fail(value->d.list_data.type != GCONF_VALUE_INVALID);
-  g_return_if_fail((list == NULL) ||
-                   ((list->data != NULL) &&
-                    (((GConfValue*)list->data)->type == value->d.list_data.type)));
+  GConfRealValue *real;
   
-  if (value->d.list_data.list)
-    gconf_value_free_list(value);
+  g_return_if_fail (value != NULL);
+  g_return_if_fail (value->type == GCONF_VALUE_LIST);
 
-  value->d.list_data.list = copy_value_list(list);
+  real = REAL_VALUE (value);
+
+  g_return_if_fail (real->d.list_data.type != GCONF_VALUE_INVALID);
+  g_return_if_fail ((list == NULL) ||
+                    ((list->data != NULL) &&
+                     (((GConfValue*)list->data)->type == real->d.list_data.type)));
+  
+  if (real->d.list_data.list)
+    gconf_value_free_list (value);
+
+  real->d.list_data.list = copy_value_list (list);
 }
 
 
 static int
-null_safe_strcmp(const char* lhs, const char* rhs)
+null_safe_strcmp (const char *lhs,
+                  const char *rhs)
 {
   if (lhs == NULL && rhs == NULL)
     return 0;
@@ -1168,6 +1369,30 @@ gconf_meta_info_free(GConfMetaInfo* gcmi)
   g_free(gcmi);
 }
 
+const char*
+gconf_meta_info_get_schema (GConfMetaInfo *gcmi)
+{
+  g_return_val_if_fail (gcmi != NULL, NULL);
+
+  return gcmi->schema;
+}
+
+const char*
+gconf_meta_info_get_mod_user (GConfMetaInfo *gcmi)
+{
+  g_return_val_if_fail (gcmi != NULL, NULL);
+
+  return gcmi->mod_user;
+}
+
+GTime
+gconf_meta_info_mod_time (GConfMetaInfo *gcmi)
+{
+  g_return_val_if_fail (gcmi != NULL, 0);
+  
+  return gcmi->mod_time;
+}
+
 void
 gconf_meta_info_set_schema  (GConfMetaInfo* gcmi,
                               const gchar* schema_name)
@@ -1214,19 +1439,41 @@ gconf_entry_new_nocopy (char* key, GConfValue* val)
   pair->schema_name = NULL;
   pair->is_default = FALSE;
   pair->is_writable = TRUE;
+  pair->refcount = 1;
   
   return pair;
 }
 
 void
-gconf_entry_free (GConfEntry* pair)
+gconf_entry_ref (GConfEntry *entry)
 {
-  g_free (pair->key);
-  if (pair->value)
-    gconf_value_free (pair->value);
-  if (pair->schema_name)
-    g_free (pair->schema_name);
-  g_free(pair);
+  g_return_if_fail (entry != NULL);
+  entry->refcount += 1;
+}
+
+void
+gconf_entry_unref (GConfEntry *entry)
+{
+  g_return_if_fail (entry != NULL);
+  g_return_if_fail (entry->refcount > 0);
+
+  entry->refcount -= 1;
+
+  if (entry->refcount == 0)
+    {
+      g_free (entry->key);
+      if (entry->value)
+        gconf_value_free (entry->value);
+      if (entry->schema_name)
+        g_free (entry->schema_name);
+      g_free (entry);
+    }
+}
+
+void
+gconf_entry_free (GConfEntry *entry)
+{
+  gconf_entry_unref (entry);
 }
 
 GConfEntry*
@@ -1283,6 +1530,47 @@ gconf_entry_steal_value (GConfEntry* entry)
   return val;
 }
 
+const char*
+gconf_entry_get_key (const GConfEntry *entry)
+{
+  g_return_val_if_fail (entry != NULL, NULL);
+
+  return entry->key;
+}
+
+GConfValue*
+gconf_entry_get_value (const GConfEntry *entry)
+{
+  g_return_val_if_fail (entry != NULL, NULL);
+
+  return entry->value;
+}
+
+const char*
+gconf_entry_get_schema_name (const GConfEntry *entry)
+{
+  g_return_val_if_fail (entry != NULL, NULL);
+
+  return entry->schema_name;
+}
+
+gboolean
+gconf_entry_get_is_default  (const GConfEntry *entry)
+{
+  g_return_val_if_fail (entry != NULL, FALSE);
+
+  return entry->is_default;
+}
+
+gboolean
+gconf_entry_get_is_writable (const GConfEntry *entry)
+{
+  g_return_val_if_fail (entry != NULL, FALSE);
+
+  return entry->is_writable;
+}
+
+
 void
 gconf_entry_set_value (GConfEntry  *entry,
                        const GConfValue  *val)
@@ -1330,11 +1618,15 @@ gboolean
 gconf_value_validate (const GConfValue *value,
                       GError          **err)
 {
+  GConfRealValue *real;
+
+  real = REAL_VALUE (value);
+  
   switch (value->type)
     {
     case GCONF_VALUE_STRING:
-      if (value->d.string_data &&
-          !g_utf8_validate (value->d.string_data, -1, NULL))
+      if (real->d.string_data &&
+          !g_utf8_validate (real->d.string_data, -1, NULL))
         {
           g_set_error (err, GCONF_ERROR,
                        GCONF_ERROR_FAILED,
@@ -1344,8 +1636,8 @@ gconf_value_validate (const GConfValue *value,
       break;
 
     case GCONF_VALUE_SCHEMA:
-      if (value->d.schema_data)
-        return gconf_schema_validate (value->d.schema_data,
+      if (real->d.schema_data)
+        return gconf_schema_validate (real->d.schema_data,
                                       err);
       break;
 
