@@ -142,7 +142,7 @@ static ConfigDatabase gconf_engine_get_database (GConfEngine     *conf,
 
 #define CHECK_OWNER_USE(engine)   \
   do { if ((engine)->owner && (engine)->owner_use_count == 0) \
-     g_warning ("You can't use a GConfEngine that has an active GConfClient wrapper object. Use GConfClient API instead.");  \
+     g_warning ("%s: You can't use a GConfEngine that has an active GConfClient wrapper object. Use GConfClient API instead.", G_GNUC_FUNCTION);  \
   } while (0)
 
 static void         register_engine           (GConfEngine    *conf);
@@ -1332,8 +1332,6 @@ gconf_engine_associate_schema  (GConfEngine* conf, const gchar* key,
   g_return_val_if_fail (conf != NULL, FALSE);
   g_return_val_if_fail (key != NULL, FALSE);
   g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
-
-  CHECK_OWNER_USE (conf);
   
   if (!gconf_key_check (key, err))
     return FALSE;
@@ -3006,11 +3004,8 @@ gconf_engine_get_string(GConfEngine* conf, const gchar* key,
           return deflt ? g_strdup(deflt) : NULL;
         }
 
-      retval = val->d.string_data;
-      /* This is a cheat; don't copy */
-      val->d.string_data = NULL; /* don't delete the string */
-
-      gconf_value_free(val);
+      retval = gconf_value_steal_string (val);
+      gconf_value_free (val);
 
       return retval;
     }
@@ -3076,12 +3071,8 @@ gconf_engine_get_schema  (GConfEngine* conf, const gchar* key, GError** err)
           return NULL;
         }
 
-      retval = gconf_value_get_schema(val);
-
-      /* This is a cheat; don't copy */
-      val->d.schema_data = NULL; /* don't delete the schema */
-
-      gconf_value_free(val);
+      retval = gconf_value_steal_schema (val);
+      gconf_value_free (val);
 
       return retval;
     }
@@ -3469,7 +3460,7 @@ gconf_string_to_enum (GConfEnumStringPair lookup_table[],
   
   while (lookup_table[i].str != NULL)
     {
-      if (g_strcasecmp(lookup_table[i].str, str) == 0)
+      if (g_ascii_strcasecmp (lookup_table[i].str, str) == 0)
         {
           *enum_value_retloc = lookup_table[i].enum_value;
           return TRUE;
@@ -3497,4 +3488,3 @@ gconf_enum_to_string (GConfEnumStringPair lookup_table[],
 
   return NULL;
 }
-
