@@ -51,6 +51,7 @@
 #include <ctype.h>
 #include <syslog.h>
 #include <time.h>
+#include <sys/wait.h>
 
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
@@ -107,6 +108,8 @@ static GConfDatabase*       obtain_database (const gchar *address,
 static void                 drop_old_databases (void);
 static gboolean             no_databases_in_use (void);
 
+static void gconf_handle_segv (int signum);
+
 /*
  * Flag indicating that we are shutting down, so return errors
  * on any attempted operation. We do this instead of unregistering with
@@ -358,15 +361,17 @@ signal_handler (int signo)
   
   ++in_fatal;
   
-  switch(signo) {
+  switch (signo) {
     /* Fast cleanup only */
   case SIGSEGV:
   case SIGBUS:
   case SIGILL:
     enter_shutdown ();
-    gconf_log(GCL_ERR,
-              _("Received signal %d, dumping core. Please report a GConf bug."),
-              signo);
+    gconf_log (GCL_ERR,
+               _("Received signal %d, dumping core. Please report a GConf bug."),
+               signo);
+    if (g_getenv ("DISPLAY"))
+      gconf_handle_segv (signo);
     abort ();
     break;
 
@@ -405,6 +410,8 @@ signal_handler (int signo)
     break;
 
   case SIGUSR1:
+    --in_fatal;
+    
     /* it'd be nice to log a message here but it's not very safe, so */
     gconf_log_debug_messages = !gconf_log_debug_messages;
     break;
@@ -547,7 +554,8 @@ main(int argc, char** argv)
   sigaction (SIGHUP,  &act, 0);
   sigaction (SIGSEGV, &act, 0);
   sigaction (SIGABRT, &act, 0);
-
+  sigaction (SIGUSR1,  &act, 0);
+  
   act.sa_handler = SIG_IGN;
   sigaction (SIGINT, &act, 0);
 
@@ -720,15 +728,11 @@ gconf_main(void)
 {
   GMainLoop* loop;
 
-  loop = g_main_new(TRUE);
+  loop = g_main_loop_new (NULL, TRUE);
 
   if (main_loops == NULL)
     {
-#ifdef GCONF_ENABLE_DEBUG
-      gulong timeout_len = 1000*60*1; /* 1 sec * 60 s/min * 1 min */
-#else
-      gulong timeout_len = 1000*60*15; /* 1 sec * 60 s/min * 2 min */
-#endif
+      gulong timeout_len = 1000*60*2; /* 1 sec * 60 s/min * 2 min */
       
       g_assert(timeout_id == 0);
       timeout_id = g_timeout_add (timeout_len,
@@ -739,7 +743,7 @@ gconf_main(void)
   
   main_loops = g_slist_prepend(main_loops, loop);
 
-  g_main_run(loop);
+  g_main_loop_run (loop);
 
   main_loops = g_slist_remove(main_loops, loop);
 
@@ -750,7 +754,7 @@ gconf_main(void)
       timeout_id = 0;
     }
   
-  g_main_destroy(loop);
+  g_main_loop_unref (loop);
 }
 
 static void 
@@ -758,7 +762,7 @@ gconf_main_quit(void)
 {
   g_return_if_fail(main_loops != NULL);
 
-  g_main_quit(main_loops->data);
+  g_main_loop_quit (main_loops->data);
 }
 
 static gboolean
@@ -1056,7 +1060,7 @@ gconf_set_exception(GError** error,
     CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
                         ex_ConfigException, ce);
 
-    gconf_log(GCL_ERR, _("Returning exception: %s"), (*error)->message);
+    gconf_log(GCL_DEBUG, _("Returning exception: %s"), (*error)->message);
       
     g_error_free(*error);
     *error = NULL;
@@ -2278,3 +2282,85 @@ client_count (void)
   else
     return g_hash_table_size (client_table);
 }
+
+
+static void
+gconf_handle_segv (int signum)
+{
+  static int in_segv = 0;
+  pid_t pid;
+	
+  in_segv++;
+
+  if (in_segv > 2)
+    {
+      /* The fprintf() was segfaulting, we are just totally hosed */
+      _exit(1);
+    }
+  else if (in_segv > 1)
+    {
+      /* dialog display isn't working out */
+      fprintf (stderr, _("Multiple segmentation faults occurred; can't display error dialog\n"));
+      _exit(1);
+    }
+        
+  pid = fork();
+
+  if (pid < 0)
+    {
+      /* Eeeek! Can't show dialog */
+      fprintf (stderr, _("Segmentation fault!\n"
+                         "Cannot display crash dialog: %s\n"),
+               strerror (errno));
+      fflush (stderr);
+      
+      /* Don't use app attributes here - a lot of things are probably hosed */
+      if (g_getenv ("GNOME_DUMP_CORE"))
+        abort ();
+      
+      _exit (1);
+    }
+  else if (pid > 0)
+    {
+      /* Wait for user to see the dialog, then exit. */
+      /* Why wait at all? Because we want to allow people to attach to the
+         process */
+      int estatus;
+      pid_t eret;
+      
+      eret = waitpid (pid, &estatus, 0);
+      
+      /* Don't use app attributes here - a lot of things are probably hosed */
+      if(g_getenv ("GNOME_DUMP_CORE"))
+        abort ();
+      
+      _exit (1);
+    }
+  else /* pid == 0 */
+    {
+      char buf[32];
+      
+      g_snprintf (buf, sizeof (buf), "%d", signum);
+      
+      /* Child process */
+      execl (GCONF_BINDIR "/gnome_segv2", GCONF_BINDIR "/gnome_segv2",
+             GCONFD, buf,
+             VERSION, NULL);
+      
+      execlp ("gnome_segv2", "gnome_segv2",
+              GCONFD, buf,
+              GCONFD, NULL);
+      
+      _exit (99);
+    }
+  
+  in_segv--;
+}
+
+
+
+
+
+
+
+
